@@ -7,19 +7,22 @@ require_once('/docker/whp/web/libs/mysqlmgmt.php');
 require_once('BackupTarget.php');
 require_once('BackupHistory.php');
 require_once('BackupStorage.php');
+require_once('BackupLocalStorage.php');
 
 use Exception;
 
 class BackupJob {
     private $db;
     private $tempDir;
-    
+    private $localStorage;
+
     public function __construct() {
         $mysql = new \mysqlmgmt();
         $this->db = $mysql->getMySQLConnection();
         $this->db->exec("USE whp");
         $this->tempDir = '/docker/tmp';
-        
+        $this->localStorage = new BackupLocalStorage($this->db);
+
         if (!is_dir($this->tempDir)) {
             mkdir($this->tempDir, 0755, true);
         }
@@ -31,99 +34,232 @@ class BackupJob {
 
         $history = new BackupHistory($this->db, $backupId);
         $data = $history->getData();
-        
+
         if ($data['status'] !== 'pending' && $data['status'] !== 'claimed') {
             throw new Exception("Backup is not in pending or claimed status, current status: " . $data['status']);
         }
-        
-        // Update status to running
+
+        // Update status to running and creation_status to creating
         $history->updateStatus('running');
-        
+        $this->updateCreationStatus($backupId, 'creating');
+
+        try {
+            // Phase 1: Create backup locally
+            $localPath = $this->createBackupLocally($data);
+
+            // Phase 2: Queue for upload
+            $this->queueForUpload($backupId, $localPath);
+
+            // Mark creation as completed (backup is now ready for upload)
+            $this->updateCreationStatus($backupId, 'created');
+            $history->updateStatus('completed'); // This means creation completed, not full upload
+
+        } catch (Exception $e) {
+            // Log the failure before updating status
+            error_log("Backup creation job {$backupId} failed: " . $e->getMessage());
+
+            // Update status to failed
+            $this->updateCreationStatus($backupId, 'failed');
+            $history->updateStatus('failed', $e->getMessage());
+
+            // Clean up any partial local backup
+            if (isset($localPath) && file_exists($localPath)) {
+                $this->localStorage->markFailed($localPath);
+            }
+
+            throw $e;
+        }
+    }
+
+    /**
+     * Create backup locally without uploading
+     */
+    private function createBackupLocally($data) {
+        // Estimate backup size for space allocation
+        $estimatedSize = $this->estimateBackupSize($data);
+
+        // Allocate local storage space
+        $localPath = $this->localStorage->allocateSpace($data['backup_name'], $estimatedSize);
+
+        // Update database with local path
+        $stmt = $this->db->prepare("UPDATE backup_history SET local_path = ? WHERE id = ?");
+        $stmt->execute([$localPath, $data['id']]);
+
         try {
             switch ($data['backup_type']) {
                 case 'site':
-                    $this->processSiteBackup($history, $data);
+                    $this->createSiteBackup($data, $localPath);
                     break;
                 case 'userfiles':
-                    $this->processUserfilesBackup($history, $data);
+                    $this->createUserfilesBackup($data, $localPath);
                     break;
                 case 'database':
-                    $this->processDatabaseBackup($history, $data);
+                    $this->createDatabaseBackup($data, $localPath);
                     break;
                 case 'wordpress':
-                    $this->processWordPressBackup($history, $data);
+                    $this->createWordPressBackup($data, $localPath);
                     break;
                 default:
                     throw new Exception("Unknown backup type: " . $data['backup_type']);
             }
-            
-            // Update status to completed
-            $history->updateStatus('completed');
-            
+
+            // Get actual file size and mark as created
+            $actualSize = filesize($localPath);
+            $readyPath = $this->localStorage->markCreated($localPath, $actualSize);
+
+            // Update database with final path and size
+            $stmt = $this->db->prepare("
+                UPDATE backup_history
+                SET local_path = ?, local_size = ?, backup_size = ?
+                WHERE id = ?
+            ");
+            $stmt->execute([$readyPath, $actualSize, $actualSize, $data['id']]);
+
+            return $readyPath;
+
         } catch (Exception $e) {
-            // Update status to failed
-            $history->updateStatus('failed', $e->getMessage());
+            // Clean up failed backup
+            $this->localStorage->markFailed($localPath);
             throw $e;
         }
     }
+
+    /**
+     * Queue backup for upload
+     */
+    private function queueForUpload($backupId, $localPath) {
+        // Add to upload queue
+        $stmt = $this->db->prepare("
+            INSERT INTO backup_upload_queue (backup_id, priority, status)
+            VALUES (?, 5, 'queued')
+            ON DUPLICATE KEY UPDATE
+            status = 'queued', retry_count = 0, error_message = NULL
+        ");
+        $stmt->execute([$backupId]);
+
+        // Update backup status
+        $stmt = $this->db->prepare("
+            UPDATE backup_history
+            SET upload_status = 'pending'
+            WHERE id = ?
+        ");
+        $stmt->execute([$backupId]);
+
+        error_log("Backup {$backupId} created locally ({$localPath}) and queued for upload");
+    }
+
+    /**
+     * Update creation status
+     */
+    private function updateCreationStatus($backupId, $status) {
+        $stmt = $this->db->prepare("
+            UPDATE backup_history
+            SET creation_status = ?
+            WHERE id = ?
+        ");
+        $stmt->execute([$status, $backupId]);
+    }
+
+    /**
+     * Estimate backup size for space allocation
+     */
+    private function estimateBackupSize($data) {
+        $metadata = is_array($data['metadata']) ? $data['metadata'] : json_decode($data['metadata'], true);
+
+        switch ($data['backup_type']) {
+            case 'site':
+            case 'wordpress':
+                // Estimate based on directory size
+                $sourcePath = $metadata['source_path'] ?? null;
+                if ($sourcePath && is_dir($sourcePath)) {
+                    return $this->getDirectorySize($sourcePath);
+                }
+                break;
+
+            case 'userfiles':
+                $sourcePath = $metadata['source_path'] ?? null;
+                if ($sourcePath && is_dir($sourcePath)) {
+                    return $this->getDirectorySize($sourcePath);
+                }
+                break;
+
+            case 'database':
+                // Estimate database size
+                $database = $metadata['database'] ?? '';
+                return $this->estimateDatabaseSize($database);
+        }
+
+        // Default estimate: 100MB
+        return 100 * 1024 * 1024;
+    }
+
+    private function getDirectorySize($directory) {
+        $size = 0;
+        try {
+            $iterator = new \RecursiveIteratorIterator(
+                new \RecursiveDirectoryIterator($directory, \RecursiveDirectoryIterator::SKIP_DOTS)
+            );
+
+            foreach ($iterator as $file) {
+                if ($file->isFile()) {
+                    $size += $file->getSize();
+                }
+            }
+        } catch (Exception $e) {
+            // If we can't read the directory, use a default estimate
+            error_log("Failed to calculate directory size for {$directory}: " . $e->getMessage());
+            $size = 50 * 1024 * 1024; // 50MB default
+        }
+
+        return $size;
+    }
+
+    private function estimateDatabaseSize($database) {
+        try {
+            $stmt = $this->db->prepare("
+                SELECT ROUND(SUM(data_length + index_length), 0) as size
+                FROM information_schema.tables
+                WHERE table_schema = ?
+            ");
+            $stmt->execute([$database]);
+            $result = $stmt->fetch();
+
+            return $result['size'] ?? (10 * 1024 * 1024); // 10MB default
+        } catch (Exception $e) {
+            error_log("Failed to estimate database size for {$database}: " . $e->getMessage());
+            return 10 * 1024 * 1024; // 10MB default
+        }
+    }
     
-    private function processSiteBackup($history, $data) {
+    private function createSiteBackup($data, $localPath) {
         $metadata = is_array($data['metadata']) ? $data['metadata'] : json_decode($data['metadata'], true);
         $sourcePath = $metadata['source_path'];
         $user = $data['user'];
         $domain = $metadata['domain'];
-        
-        // Step 1: Create archive
-        $tempFile = $this->tempDir . '/' . $data['backup_name'] . '.tar.gz';
-        $this->createSiteArchive($sourcePath, $tempFile, $user, $domain);
-        
-        // Step 2: Upload to storage
-        $this->uploadToStorage($history, $data, $tempFile);
-        
-        // Step 3: Clean up temp file
-        if (file_exists($tempFile)) {
-            unlink($tempFile);
-        }
+
+        // Create archive directly to local storage
+        $this->createSiteArchive($sourcePath, $localPath, $user, $domain);
     }
     
-    private function processUserfilesBackup($history, $data) {
+    private function createUserfilesBackup($data, $localPath) {
         $metadata = is_array($data['metadata']) ? $data['metadata'] : json_decode($data['metadata'], true);
         $sourcePath = $metadata['source_path'];
         $user = $data['user'];
-        
-        // Step 1: Create archive
-        // Try zip format for better compatibility
-        $tempFile = $this->tempDir . '/' . $data['backup_name'] . '.zip';
-        $this->createUserfilesArchiveZip($sourcePath, $tempFile, $user);
-        
-        // Step 2: Upload to storage
-        $this->uploadToStorage($history, $data, $tempFile);
-        
-        // Step 3: Clean up temp file
-        if (file_exists($tempFile)) {
-            unlink($tempFile);
-        }
+
+        // Create archive directly to local storage (use zip for better compatibility)
+        $this->createUserfilesArchiveZip($sourcePath, $localPath, $user);
     }
     
-    private function processDatabaseBackup($history, $data) {
+    private function createDatabaseBackup($data, $localPath) {
         $metadata = is_array($data['metadata']) ? $data['metadata'] : json_decode($data['metadata'], true);
         $database = $metadata['database'];
         $user = $data['user'];
 
-        // Step 1: Create SQL dump
-        $tempFile = $this->tempDir . '/' . $data['backup_name'] . '.sql.gz';
-        $this->createDatabaseDump($database, $tempFile);
-
-        // Step 2: Upload to storage
-        $this->uploadToStorage($history, $data, $tempFile);
-
-        // Step 3: Clean up temp file
-        if (file_exists($tempFile)) {
-            unlink($tempFile);
-        }
+        // Create SQL dump directly to local storage
+        $this->createDatabaseDump($database, $localPath);
     }
 
-    private function processWordPressBackup($history, $data) {
+    private function createWordPressBackup($data, $localPath) {
         $metadata = is_array($data['metadata']) ? $data['metadata'] : json_decode($data['metadata'], true);
         $sourcePath = $metadata['source_path'];
         $database = $metadata['database'] ?? null;
@@ -145,17 +281,8 @@ class BackupJob {
             throw new Exception("Invalid source path");
         }
 
-        // Step 1: Create combined WordPress archive
-        $tempFile = $this->tempDir . '/' . $data['backup_name'] . '.tar.gz';
-        $this->createWordPressArchive($sourcePath, $database, $tempFile, $user, $domain);
-
-        // Step 2: Upload to storage
-        $this->uploadToStorage($history, $data, $tempFile);
-
-        // Step 3: Clean up temp file
-        if (file_exists($tempFile)) {
-            unlink($tempFile);
-        }
+        // Create combined WordPress archive directly to local storage
+        $this->createWordPressArchive($sourcePath, $database, $localPath, $user, $domain);
     }
     
     private function createSiteArchive($sourcePath, $tempFile, $user, $domain) {
@@ -430,17 +557,5 @@ class BackupJob {
         rmdir($dir);
     }
 
-    private function uploadToStorage($history, $data, $tempFile) {
-        $target = new BackupTarget($this->db, $data['target_id']);
-        $storage = new BackupStorage($target->getCredentials());
-        
-        $result = $storage->uploadFile($tempFile, $data['backup_path']);
-        
-        if (!$result['success']) {
-            throw new Exception("Failed to upload backup: " . $result['error']);
-        }
-        
-        // Update backup size
-        $history->updateSize($result['size']);
-    }
+    // uploadToStorage method removed - uploads now handled by separate upload worker
 }
\ No newline at end of file
